;; NAME
;;	selection-stats
;;
;; VERSION
;;	0.2.0, 2008/12/13
;;
;; DESCRIPTION
;;	Selection-stats is a Script-Fu plugin for GIMP.  It uses GIMP's
;;	histogram function to find the current selection's perimeter in
;;	pixels and its area as a percentage of total image area.
;;
;; AUTHOR
;;	user "isomage" of the cartographersguild.com forums
;;
;; LICENSE
;;	Selection-stats is free software.

(define (selection-stats image)

  (define (fg/bg-selection drawable)
    (gimp-edit-bucket-fill drawable FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
    (gimp-selection-invert image)
    (gimp-edit-bucket-fill drawable BG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0))

  (if (= (car (gimp-selection-bounds image)) FALSE)
      (gimp-message "You must select an area of the image")

      (let* ((width     (car (gimp-image-width image)))
	     (height    (car (gimp-image-height image)))
	     (selection (car (gimp-selection-save image)))
	     (fgcolor   (car (gimp-context-get-foreground)))
	     (bgcolor   (car (gimp-context-get-background)))
	     (scratch   (car (gimp-layer-new image width height RGB-IMAGE "scratch" 100 NORMAL-MODE)))
	     (area      0)
	     (perimeter 0))

	(gimp-image-add-layer image scratch 0)

	(gimp-context-set-background '(0 0 0))
	(gimp-context-set-foreground '(255 0 0))

	(fg/bg-selection scratch)
	(gimp-selection-none image)
	(let ((histogram (gimp-histogram scratch HISTOGRAM-RED 128 255)))
	  (set! area (* 100 (list-ref histogram 5))))

	(gimp-selection-load selection)

	(gimp-selection-border image 1)

	(fg/bg-selection scratch)
	(gimp-selection-none image)
	(let ((histogram (gimp-histogram scratch HISTOGRAM-RED 128 255)))
	  (set! perimeter (list-ref histogram 4)))

	(gimp-image-remove-layer image scratch)

	(gimp-context-set-background bgcolor)
	(gimp-context-set-foreground fgcolor)

	(gimp-selection-load selection)

	(gimp-message (string-append "Area " (number->string area) "%, "
				     "Perimeter " (number->string perimeter) " pixels")))))

(script-fu-register "selection-stats"
		    "<Image>/Select/Area and perimeter"
		    "Find the perimeter of a selection in pixels and the area as a percentage of image area"
		    "isomage"
		    "isomage"
		    "Created 2008/12/13, version 0.2.0 2008/12/13"
		    ""

		    SF-IMAGE "Image" 0
)
